import { Type } from "@manta-style/core";
import MantaStyle from "@manta-style/runtime";
/**
 * Make all properties in T optional
 * @preserveUnion
 */
export type Partial<T> = { [P in keyof T]?: T[P] };
export declare const Partial: Type<Partial<any>>;


/**
 * Make all properties in T required
 * @preserveUnion
 */
export type Required<T> = { [P in keyof T]-?: T[P] };
export declare const Required: Type<Required<any>>;


/**
 * Make all properties in T readonly
 * @preserveUnion
 */
export type Readonly<T> = { readonly [P in keyof T]: T[P] };
export declare const Readonly: Type<Readonly<any>>;


/**
 * From T pick a set of properties K
 * @preserveUnion
 */
export type Pick<T, K extends keyof T> = { [P in K]: T[P] };
export declare const Pick: Type<Pick<any,any>>;


/**
 * Construct a type with a set of properties K of type T
 * @preserveUnion
 */
export type Record<K extends keyof any, T> = { [P in K]: T };
export declare const Record: Type<Record<any,any>>;


/**
 * Exclude from T those export types that are assignable to U
 * @preserveUnion
 */
export type Exclude<T, U> = T extends U ? never : T;
export declare const Exclude: Type<Exclude<any,any>>;


/**
 * Extract from T those export types that are assignable to U
 * @preserveUnion
 */
export type Extract<T, U> = T extends U ? T : never;
export declare const Extract: Type<Extract<any,any>>;


/**
 * Exclude null and undefined from T
 * @preserveUnion
 */
export type NonNullable<T> = T extends null | undefined ? never : T;
export declare const NonNullable: Type<NonNullable<any>>;


/**
 * @preserveUnion
 */
export type Array<T> = T[];
export declare const Array: Type<Array<any>>;


/**
 * @preserveUnion
 */
export type ReadonlyArray<T> = T[];
export declare const ReadonlyArray: Type<ReadonlyArray<any>>;