"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ReadonlyArray = exports.Array = exports.NonNullable = exports.Extract = exports.Exclude = exports.Record = exports.Pick = exports.Readonly = exports.Required = exports.Partial = void 0;

var _runtime = _interopRequireDefault(require("@manta-style/runtime"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Make all properties in T optional
 * @preserveUnion
 */
var Partial = _runtime.default.TypeAliasDeclaration("Partial", function (typeFactory) {
  var T = typeFactory.TypeParameter("T");

  var type = _runtime.default.MappedType(function (mappedType) {
    var P = mappedType.TypeParameter("P", _runtime.default.KeyOfKeyword(T));
    mappedType.setQuestionToken(1);
    mappedType.setConstraint(_runtime.default.KeyOfKeyword(T));
    return _runtime.default.IndexedAccessType(T, P);
  });

  return type;
}, [{
  key: "preserveUnion",
  value: ""
}]);
/**
 * Make all properties in T required
 * @preserveUnion
 */


exports.Partial = Partial;

var Required = _runtime.default.TypeAliasDeclaration("Required", function (typeFactory) {
  var T = typeFactory.TypeParameter("T");

  var type = _runtime.default.MappedType(function (mappedType) {
    var P = mappedType.TypeParameter("P", _runtime.default.KeyOfKeyword(T));
    mappedType.setQuestionToken(2);
    mappedType.setConstraint(_runtime.default.KeyOfKeyword(T));
    return _runtime.default.IndexedAccessType(T, P);
  });

  return type;
}, [{
  key: "preserveUnion",
  value: ""
}]);
/**
 * Make all properties in T readonly
 * @preserveUnion
 */


exports.Required = Required;

var Readonly = _runtime.default.TypeAliasDeclaration("Readonly", function (typeFactory) {
  var T = typeFactory.TypeParameter("T");

  var type = _runtime.default.MappedType(function (mappedType) {
    var P = mappedType.TypeParameter("P", _runtime.default.KeyOfKeyword(T));
    mappedType.setQuestionToken(0);
    mappedType.setConstraint(_runtime.default.KeyOfKeyword(T));
    return _runtime.default.IndexedAccessType(T, P);
  });

  return type;
}, [{
  key: "preserveUnion",
  value: ""
}]);
/**
 * From T pick a set of properties K
 * @preserveUnion
 */


exports.Readonly = Readonly;

var Pick = _runtime.default.TypeAliasDeclaration("Pick", function (typeFactory) {
  var T = typeFactory.TypeParameter("T");
  var K = typeFactory.TypeParameter("K", _runtime.default.KeyOfKeyword(T));

  var type = _runtime.default.MappedType(function (mappedType) {
    var P = mappedType.TypeParameter("P", K);
    mappedType.setQuestionToken(0);
    mappedType.setConstraint(K);
    return _runtime.default.IndexedAccessType(T, P);
  });

  return type;
}, [{
  key: "preserveUnion",
  value: ""
}]);
/**
 * Construct a type with a set of properties K of type T
 * @preserveUnion
 */


exports.Pick = Pick;

var Record = _runtime.default.TypeAliasDeclaration("Record", function (typeFactory) {
  var K = typeFactory.TypeParameter("K", _runtime.default.KeyOfKeyword(_runtime.default.AnyKeyword));
  var T = typeFactory.TypeParameter("T");

  var type = _runtime.default.MappedType(function (mappedType) {
    var P = mappedType.TypeParameter("P", K);
    mappedType.setQuestionToken(0);
    mappedType.setConstraint(K);
    return T;
  });

  return type;
}, [{
  key: "preserveUnion",
  value: ""
}]);
/**
 * Exclude from T those export types that are assignable to U
 * @preserveUnion
 */


exports.Record = Record;

var Exclude = _runtime.default.TypeAliasDeclaration("Exclude", function (typeFactory) {
  var T = typeFactory.TypeParameter("T");
  var U = typeFactory.TypeParameter("U");

  var type = _runtime.default.ConditionalType(T, U, _runtime.default.NeverKeyword, T);

  return type;
}, [{
  key: "preserveUnion",
  value: ""
}]);
/**
 * Extract from T those export types that are assignable to U
 * @preserveUnion
 */


exports.Exclude = Exclude;

var Extract = _runtime.default.TypeAliasDeclaration("Extract", function (typeFactory) {
  var T = typeFactory.TypeParameter("T");
  var U = typeFactory.TypeParameter("U");

  var type = _runtime.default.ConditionalType(T, U, T, _runtime.default.NeverKeyword);

  return type;
}, [{
  key: "preserveUnion",
  value: ""
}]);
/**
 * Exclude null and undefined from T
 * @preserveUnion
 */


exports.Extract = Extract;

var NonNullable = _runtime.default.TypeAliasDeclaration("NonNullable", function (typeFactory) {
  var T = typeFactory.TypeParameter("T");

  var type = _runtime.default.ConditionalType(T, _runtime.default.UnionType([_runtime.default.NullKeyword, _runtime.default.UndefinedKeyword]), _runtime.default.NeverKeyword, T);

  return type;
}, [{
  key: "preserveUnion",
  value: ""
}]);
/**
 * @preserveUnion
 */


exports.NonNullable = NonNullable;

var Array = _runtime.default.TypeAliasDeclaration("Array", function (typeFactory) {
  var T = typeFactory.TypeParameter("T");

  var type = _runtime.default.ArrayType(T);

  return type;
}, [{
  key: "preserveUnion",
  value: ""
}]);
/**
 * @preserveUnion
 */


exports.Array = Array;

var ReadonlyArray = _runtime.default.TypeAliasDeclaration("ReadonlyArray", function (typeFactory) {
  var T = typeFactory.TypeParameter("T");

  var type = _runtime.default.ArrayType(T);

  return type;
}, [{
  key: "preserveUnion",
  value: ""
}]);

exports.ReadonlyArray = ReadonlyArray;